package mactrans.system;

import mactrans.database.*;
import mactrans.parser.*;
import mactrans.alignment.*;
import mactrans.matching.SignBased.*;
import mactrans.matching.aos.*;
import mactrans.matching.ed.*;
import mactrans.recombination.*;

/**
 * 
 * @author Jan Kienecker (3kieneck@informatik.uni-hamburg.de)
 * @version $Revision: 1.13 $
 * 
 * <pre>
 * $Log: MachineTranslationClass.java,v $
 * Revision 1.13  2005/02/14 18:51:49  dfriehe
 * adapted for recombination
 *
 * Revision 1.12  2005/02/11 17:51:37  dfriehe
 * improved w/r null-pointer handling
 *
 * Revision 1.11  2005/02/04 15:42:35  dfriehe
 * various
 *
 * Revision 1.10  2005/02/02 13:53:24  jkieneck
 * :-)
 *
 * Revision 1.9  2005/02/01 15:35:31  dfriehe
 * added SampleParser (globally accessible by now)
 *
 * Revision 1.8  2005/01/26 19:34:36  jkieneck
 * updated to fit for current progress (EditDistance contributed)
 *
 * Revision 1.7  2005/01/26 13:43:20  jkieneck
 * updated for first "real" actions
 *
 * Revision 1.6  2005/01/26 08:24:07  jkieneck
 * embedded getResultOf, internal callProcess etc...
 *
 * Revision 1.5  2005/01/24 04:50:14  jkieneck
 * added support for recombination
 *
 * Revision 1.4  2005/01/19 13:19:26  jkieneck
 * adapted to new interface
 *
 * Revision 1.3  2005/01/05 13:27:26  jkieneck
 * added further methods and corrected old stuff
 *
 * Revision 1.2  2005/01/05 12:26:37  jkieneck
 * updated to project status (A. Weidacher left)
 *
 * Revision 1.1.1.1  2004/12/14 23:28:03  mactrans
 * added to repository
 *
 * </pre>
 */

public abstract class MachineTranslationClass extends AbstractMachineTranslationClass {
	
	/**
	 * For your comfort..
	 * 
	 * @param p MTP object produced a result
	 * @return MTP's result, if
	 * @see Results#getResultOf
	 */
	public Object getResultOf( MachineTranslationProcessor p ) {
		return results().getResultOf( p );
	}
	
	/**
	 * Calls an MTP and stores the result
	 * 
	 * @param p MTP object to be called
	 */
	public void callProcessor( MachineTranslationProcessor p ) {
		if ( isInitialized( p ) ) {
			p.initialize();
			Object o = p.proceed();
			if ( o == null ) {
				system().error( "MachineTranslationClass", "callProcessor: the Processor returned NULL" );
			}
			results().storeResultOf( p, o );
		} else {
			system().fatalError( "MachineTranslationClass", "callProcessor; processor not initialized" );
		}
	}
	
	/**
	 * Returns the current algorithm used
	 * 
	 * @return A defined algorithm
	 */
	public MachineTranslationProcessor getMatchingProcessor() {
		return MachineTranslationKit._algorithm;
	}
	
	/**
	 * 
	 * @return access to the input module
	 */
	public IOBase input() {
		IOBase result = null;
		if ( isInitialized( MachineTranslationKit._input ) ) {
			result = MachineTranslationKit._input;
		} else {
			system().fatalError( "MachineTranslationClass", "input(IOBase) not initialized" );
		}
		return result;
	}

	/**
	 * 
	 * @return access to the output module
	 */
	public IOBase output() {
		IOBase result = null;
		if ( isInitialized( MachineTranslationKit._output ) ) {
			result = MachineTranslationKit._output;
		} else {
			system().fatalError( "MachineTranslationClass", "output(IOBase) not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the results module
	 */
	public Results results() {
		Results result = null;
		if ( isInitialized( MachineTranslationKit._results ) ) {
			result = MachineTranslationKit._results;
		} else {
			system().fatalError( "MachineTranslationClass", "results(Results) not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the German sentenceDatabase module
	 */
	public SentenceDatabase sentenceDatabaseDE() {
		SentenceDatabase result = null;
		if ( isInitialized( MachineTranslationKit._sentenceDatabaseDE ) ) {
			result = MachineTranslationKit._sentenceDatabaseDE;
		} else {
			system().fatalError( "MachineTranslationClass", "sentenceDatabaseDE(SentenceDatabase) not initialized" );
		}
		return result;
	}

	/**
	 * 
	 * @return access to the English sentenceDatabase module
	 */
	public SentenceDatabase sentenceDatabaseEN() {
		SentenceDatabase result = null;
		if ( isInitialized( MachineTranslationKit._sentenceDatabaseEN ) ) {
			result = MachineTranslationKit._sentenceDatabaseEN;
		} else {
			system().fatalError( "MachineTranslationClass", "sentenceDatabaseEN(SentenceDatabase) not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @param lang language, either DE or EN
	 * @return access to sentenceDatabase module spec'd by lang
	 */
	public SentenceDatabase sentenceDatabase( String lang ) {
		SentenceDatabase result = null;
		if ( lang.equals( "DE" ) ) {
			result = sentenceDatabaseDE();
		} else {
			if ( lang.equals( "EN" ) ) {
				result = sentenceDatabaseEN();
			} else {
				system().fatalError( "MachineTranslationClass", "sentenceDatabase(SentenceDatabase): unknown language '" + lang + "'" );
			}
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the German dictionary module
	 */
	public Dictionary dictionaryDE() {
		Dictionary result = null;
		if ( isInitialized( MachineTranslationKit._dictionaryDE ) ) {
			result = MachineTranslationKit._dictionaryDE;
		} else {
			system().fatalError( "MachineTranslationClass", "dictionaryDE(Dictionary) not initialized" );
		}
		return result;
	}

	/**
	 * 
	 * @return access to the English dictionary module
	 */
	public Dictionary dictionaryEN() {
		Dictionary result = null;
		if ( isInitialized( MachineTranslationKit._dictionaryEN ) ) {
			result = MachineTranslationKit._dictionaryEN;
		} else {
			system().fatalError( "MachineTranslationClass", "dictionaryEN(Dictionary) not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @param lang language, either DE or EN
	 * @return access to dictionary module spec'd by lang
	 */
	public Dictionary dictionary( String lang ) {
		Dictionary result = null;
		if ( lang.equals( "DE" ) ) {
			result = dictionaryDE();
		} else {
			if ( lang.equals( "EN" ) ) {
				result = dictionaryEN();
			} else {
				system().fatalError( "MachineTranslationClass", "dictionary(Dictionary): unknown language '" + lang + "'" );
			}
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the German grammar module
	 */
	public AbstractGrammar grammarDE() {
		AbstractGrammar result = null;
		if ( isInitialized( MachineTranslationKit._grammarDE ) ) {
			result = MachineTranslationKit._grammarDE;
		} else {
			system().fatalError( "MachineTranslationClass", "grammarDE(Grammar) not initialized" );
		}
		return result;
	}

	/**
	 * 
	 * @return access to the English grammar module
	 */
	public AbstractGrammar grammarEN() {
		AbstractGrammar result = null;
		if ( isInitialized( MachineTranslationKit._grammarEN ) ) {
			result = MachineTranslationKit._grammarEN;
		} else {
			system().fatalError( "MachineTranslationClass", "grammarEN(Grammar) not initialized" );
		}
		return result;
	}

	/**
	 * 
	 * @param lang language, either DE or EN
	 * @return access to the grammar module spec'd by lang
	 */
	public AbstractGrammar grammar( String lang ) {
		AbstractGrammar result = null;
		if ( lang.equals( "DE" ) ) {
			result = grammarDE();
		} else {
			if ( lang.equals( "EN" ) ) {
				result = grammarEN();
			} else {
				system().fatalError( "MachineTranslationClass", "grammar(Grammar): unknown language '" + lang + "'" );
			}
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the German parser module
	 */
	public Parser parserDE() {
		Parser result = null;
		if ( isInitialized( MachineTranslationKit._parserDE ) ) {
			result = MachineTranslationKit._parserDE;
		} else {
			system().fatalError( "MachineTranslationClass", "parserDE(Parser) not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the English parser module
	 */
	public Parser parserEN() {
		Parser result = null;
		if ( isInitialized( MachineTranslationKit._parserEN ) ) {
			result = MachineTranslationKit._parserEN;
		} else {
			system().fatalError( "MachineTranslationClass", "parserEN(Parser) not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @param lang language, either DE or EN
	 * @return access to parser module spec'd by lang
	 */
	public Parser parser( String lang ) {
		Parser result = null;
		if ( lang.equals( "DE" ) ) {
			result = parserDE();
		} else {
			if ( lang.equals( "EN" ) ) {
				result = parserEN();
			} else {
				system().fatalError( "MachineTranslationClass", "parser(Parser): unknown language '" + lang + "'" );
			}
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the SampleParser module
	 */
	public SampleParser sampleParser() {
		SampleParser result = null;
		if ( isInitialized( MachineTranslationKit._sampleParser ) ) {
			result = MachineTranslationKit._sampleParser;
		} else {
			system().fatalError( "MachineTranslationClass", "sampleParser not initialized" );
		}
		return result;
	}

	/**
	 * 
	 * @return access to the SignBasedMatching module
	 */
	public SignBasedMatching signBasedMatching() {
		SignBasedMatching result = null;
		if ( isInitialized( MachineTranslationKit._signBasedMatching ) ) {
			result = MachineTranslationKit._signBasedMatching;
		} else {
			system().fatalError( "MachineTranslationClass", "signBasedMatching not initialized" );
		}
		return result;
	}
	
	// A. Weidacher left the project
	/*
	public SemanticMatching semanticMatching() {
		if ( isInitialized( MachineTranslationKit._semanticMatching ) ) {
			return MachineTranslationKit._semanticMatching;
		} else {
			system().fatalError( "semanticMatching not initialized" );
		}
	}
	*/
	
	/**
	 * 
	 * @return access to the WordMatching module
	 */
	public AoSMatching wordMatching() {
		AoSMatching result = null;
		if ( isInitialized( MachineTranslationKit._wordMatching ) ) {
			result = MachineTranslationKit._wordMatching;
		} else {
			system().fatalError( "MachineTranslationClass", "wordMatching not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the EditDistance module
	 */
	public Edit editDistance() {
		Edit result = null;
		if ( isInitialized( MachineTranslationKit._editDistance ) ) {
			result = MachineTranslationKit._editDistance;
		} else {
			system().fatalError( "MachineTranslationClass", "editDistance not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the Alignment module
	 */
	public Alignments alignment() {
		Alignments result = null;
		if ( isInitialized( MachineTranslationKit._alignment ) ) {
			result = MachineTranslationKit._alignment;
		} else {
			system().fatalError( "MachineTranslationClass", "alignment not initialized" );
		}
		return result;
	}
	
	/**
	 * 
	 * @return access to the recombination module
	 */
	public Recombination recombination() {
		Recombination result = null;
		if ( isInitialized( MachineTranslationKit._recombination ) ) {
			result = MachineTranslationKit._recombination;
		} else {
			system().fatalError( "MachineTranslationClass", "recombination(Recombination) not initialized" );
		}
		return result;
	}
}